/** Obsługa budzików lwIP w wersji >= 1.4.0 dla konfiguracji
    #define NO_SYS                  1
    #define NO_SYS_NO_TIMERS        0
**/

#include <delay.h>
#include <lwip_util.h>
#include <time_util.h>
#include <lwip/init.h>
#include <lwip/dhcp.h>
#include <lwip/sys.h>
#include <lwip/timers.h>
#include <netif/etharp.h>

/* Ta funkcja nie jest wołana w kontekście procedury
   obsługi przerwania. */
int LWIPinterfaceInit(struct netif *netif,
                      struct ethnetif *ethnetif) {
  ip_addr_t ipaddr, netmask, gw;
  struct netif *p;
  #if LWIP_DHCP
    int dhcpFlag;
  #endif
  IRQ_DECL_PROTECT(x);

  if (netif->ip_addr.addr == 0) {
    #if LWIP_DHCP
      ipaddr.addr = 0;
      netmask.addr = 0;
      gw.addr = 0;
      dhcpFlag = 1;
    #else
      return -1;
    #endif
  }
  else {
    ipaddr = netif->ip_addr;
    netmask = netif->netmask;
    gw = netif->gw;
    #if LWIP_DHCP
      dhcpFlag = 0;
    #endif
  }
  IRQ_PROTECT(x, LWIP_IRQ_PRIO);
  lwip_init();
  p = netif_add(netif,
                &ipaddr,
                &netmask,
                &gw,
                ethnetif, /* wskaźnik do prywatnych danych */
                &ETHinit, /* nasza funkcja inicjująca interfejs */
                &ethernet_input /* funkcja stosu lwIP przetwarzająca
                                   odebrane ramki ethernetowe */);
  IRQ_UNPROTECT(x);
  if (!p)
    return -1;

  IRQ_PROTECT(x, LWIP_IRQ_PRIO);
  netif_set_default(netif);
  IRQ_UNPROTECT(x);

  /* dhcp_start() i netif_set_up() ustawiają m.in.
     netif->flags |= NETIF_FLAG_UP; */
  #if LWIP_DHCP
    if (dhcpFlag) {
      err_t err;

      IRQ_PROTECT(x, LWIP_IRQ_PRIO);
      err = dhcp_start(netif);
      IRQ_UNPROTECT(x);
      if (err != ERR_OK)
        return -1;
    }
    else {
  #endif
      IRQ_PROTECT(x, LWIP_IRQ_PRIO);
      netif_set_up(netif);
      IRQ_UNPROTECT(x);
  #if LWIP_DHCP
    }
  #endif

  return 0;
}

/* SysTick ma wyższy priorytet niż lwIP - czas musi biegnąć podczas
   przetwarzania ramek. SysTick wywołuje cyklicznie funkcję zwrotną
   (ang. call back) LWIPtmr, która zgłasza przerwanie PendSV,
   a dopiero procedura obsługi tego przerwania obsługuje właściwe
   budziki w lwIP, gdyż chcemy, aby SysTick i budziki działały
   z różnymi priorytetami. */

/* Główny budzik wołany przez SysTick wyzwala przerwanie PendSV. */
static void LWIPtmr(void) {
  SCB->ICSR |= SCB_ICSR_PENDSVSET_Msk; /* Było SCB_ICSR_PENDSVSET. */
}

/* Uaktywnia główny budzik. Nie jest wołana w kontekście procedury
   obsługi przerwania - ochrona jest w funkcji TimerCallBack. */
void LWIPtimersStart() {
  IRQsetPriority(PendSV_IRQn, LWIP_IRQ_PRIO, 0);
  TimerCallBack(LWIPtmr);
}

/* Procedura obsługi przerwania PendSV wołająca budziki lwIP. */
void PendSV_Handler(void) {
  sys_check_timeouts();
}

/* Stos lwIP woła tę funkcję, aby dowiedzieć się, ile ms upłynęło. */
u32_t sys_now() {
  return LocalTime();
}

/* Ta funkcja nie jest wołana w kontekście procedury obsługi
   przerwania. Ochrona jest tu przesadna, bo tylko czytamy dane,
   ale przypomina, że zdarzenia zachodzą współbieżnie.
   Czekaj aktywnie maksymalnie timeout sekund lub tries prób. */
int DHCPwait(struct netif *netif, unsigned timeout, unsigned tries) {
  #if LWIP_DHCP
    volatile uint8_t netif_flags;
    IRQ_DECL_PROTECT(x);

    IRQ_PROTECT(x, LWIP_IRQ_PRIO);
    netif_flags = netif->flags;
    IRQ_UNPROTECT(x);
    if (netif_flags & NETIF_FLAG_DHCP) {
      ltime_t t;

      t = LocalTime() + timeout * 1000;
      while (t > LocalTime()) {
        volatile uint8_t dhcp_tries;
        volatile uint8_t dhcp_state;

        /* Nie należy sprawdzać zbyt często. */
        Delay(1000000);

        IRQ_PROTECT(x, LWIP_IRQ_PRIO);
        dhcp_tries = netif->dhcp->tries;
        dhcp_state = netif->dhcp->state;
        IRQ_UNPROTECT(x);

        if (dhcp_state == DHCP_BOUND)
          return 0;
        else if (dhcp_tries >= tries) {
          IRQ_PROTECT(x, LWIP_IRQ_PRIO);
          dhcp_stop(netif);
          IRQ_UNPROTECT(x);
          return -1;
        }
      }
      return -1;
    }
  #endif

  return 0;
}
