#ifndef _CORTEX_M_H
#define _CORTEX_M_H 1

#include <irq.h>
#include <stdlib.h>

#define BYTE_ORDER LITTLE_ENDIAN

/* Stos lwIP używa własnych nazw dla typów całościowych (ang.
   integral types). Ogólny wskaźnik mem_ptr_t nie jest zdefiniowany
   jako void*, aby możliwe były wszystkie operacje arytmetyczne. */
typedef int8_t   s8_t;
typedef uint8_t  u8_t;
typedef int16_t  s16_t;
typedef uint16_t u16_t;
typedef int32_t  s32_t;
typedef uint32_t u32_t;
typedef u32_t    mem_ptr_t;

/* Stos lwIP potrzebuje specyfikatorów formatowania dla funkcji
   *printf. Typy int16_t, uint16_t, int32_t, uint32_t są zdefiniowane
   w stdint.h odpowiednio jako short, unsigned short, long,
   unsigned long. SZ_T specyfikuje format dla typu size_t. */
#define S16_F "hd"
#define U16_F "hu"
#define X16_F "hx"
#define S32_F "ld"
#define U32_F "lu"
#define X32_F "lx"
#define SZT_F "zu"

#if defined __GNUC__
  #define ALIGN4 __attribute__ ((aligned (4)))
#else
  #error Define ALIGN4 macro for your compiler.
#endif

/* Optymalizuj operacje odwracania kolejności bajtów - bardzo
   przydatne w architekturze little-endian. */
#define LWIP_PLATFORM_HTONS(x) __REV16(x)
#define LWIP_PLATFORM_HTONL(x) __REV(x)

/* Optymalizuj czas działania - nie używaj asercji. */
#define LWIP_PLATFORM_DIAG(x) ((void)0)
#define LWIP_PLATFORM_ASSERT(x) ((void)0)
/* Oryginalnie było tak
#define LWIP_PLATFORM_ASSERT(x) do { if(!(x)) while(1); } while(0)
*/

/* Jeśli w lwiopts.h zdefiniowano opcję SYS_LIGHTWEIGHT_PROT == 1,
   trzeba też zdefiniować makra chroniące dostępy do pamięci. Makra
   te pojawiają się w funkcjach memp_free, memp_malloc, pbuf_ref,
   pbuf_free, pbuf_free_ooseq, pbuf_pool_is_empty oraz gdy
   zdefiniowano MEM_LIBC_MALLOC == 0, to również w mem_free,
   mem_malloc i mem_trim. Ponadto w netif_loop_output i netif_poll. */
#define SYS_ARCH_DECL_PROTECT(x)  irq_level_t x
#define SYS_ARCH_PROTECT(x)       x = IRQprotectAll()
#define SYS_ARCH_UNPROTECT(x)     IRQunprotectAll(x)

/* Definiujemy priorytet dla obsługi Ethernetu i TCP/IP. */
#define LWIP_IRQ_PRIO  MIDDLE_IRQ_PRIO

/* Makro IRQ_DECL_PROTECT(prv) deklaruje zmienną prv, która
   przechowuje informację o blokowanych przerwaniach. Makro
   IRQ_PROTECT(prv, lvl) zapisuje w zmiennej prv stan aktualnie
   blokowanych przerwań i blokuje przerwania o priorytecie niższym
   lub równym lvl. Makro IRQ_UNPROTECT(prv) przywraca stan blokowania
   przerwań zapisany w zmiennej prv. */
#define IRQ_DECL_PROTECT(prv)  irq_level_t prv
#define IRQ_PROTECT(prv, lvl)  prv = IRQprotect(lvl)
#define IRQ_UNPROTECT(prv)     IRQunprotect(prv)

/* W pliku lwiopts.h nazwy funkcji obsługujących alokację pamięci
   mem_* są przedefiniowane na protected_*. Tu są ich definicje.
   Wydaje się, że stos lwIP używa tylko mem_malloc i mem_free.
   Funkcje stosu wołane są w kontekście przerwania lub przy
   zablokowanych przrwaniach o priorytecie równym lub niższym niż
   LWIP_IRQ_PRIO. Ochrona jest potrzebna tylko wtedy, gdy alokacja
   pamięci odbywa się w innych przerwaniach. Nie wolno używać
   niechronionych funkcji malloc, calloc, free i realloc w programie
   głównym (w main i funkcjach wołanych bezpośrednio lub pośrednio
   przez main). */

static inline void protected_free(void *ptr) {
  SYS_ARCH_DECL_PROTECT(v);
  SYS_ARCH_PROTECT(v);
  free(ptr);
  SYS_ARCH_UNPROTECT(v);
}

static inline void *protected_calloc(size_t nmemb, size_t size) {
  void *ret;
  SYS_ARCH_DECL_PROTECT(v);
  SYS_ARCH_PROTECT(v);
  ret = calloc(nmemb, size);
  SYS_ARCH_UNPROTECT(v);
  return ret;
}

static inline void *protected_malloc(size_t size) {
  void *ret;
  SYS_ARCH_DECL_PROTECT(v);
  SYS_ARCH_PROTECT(v);
  ret = malloc(size);
  SYS_ARCH_UNPROTECT(v);
  return ret;
}

static inline void *protected_realloc(void *ptr, size_t size) {
  void *ret;
  SYS_ARCH_DECL_PROTECT(v);
  SYS_ARCH_PROTECT(v);
  ret = realloc(ptr, size);
  SYS_ARCH_UNPROTECT(v);
  return ret;
}

#endif
